@@ -9,7 +9,7 @@
 
 package Math::Formula;
 use vars '$VERSION';
-$VERSION = '0.11';
+$VERSION = '0.12';
 
 
 use warnings;
@@ -72,13 +72,10 @@ my $match_dt    = MF::DATETIME->_match;
 my $match_dur   = MF::DURATION->_match;
 
 my $match_op    = join '|',
-	'[*\/+\-#~.%]',
-	qw/=~ !~
-	 <=> <= >= == != < > /,  # order is important
-	( map "$_\\b", qw/
-		and or not xor
-		like unlike
-		cmp lt le eq ne ge gt/
+	qw{ // }, '[?*\/+\-#~.%]',
+	qw{ =~ !~ <=> <= >= == != < > },  # order is important
+	qw{ :(?![0-9][0-9]) (?<![0-9][0-9]): },
+	( map "$_\\b", qw/ and or not xor exists like unlike cmp lt le eq ne ge gt/
 	);
 
 sub _tokenize($)
@@ -126,7 +123,8 @@ sub _build_ast($$)
   PROGRESS:
 	while(my $first = shift @$t)
 	{
-#warn "LOOP FIRST ", Dumper $first,
+#use Data::Dumper; $Data::Dumper::Indent = 0;
+#warn "LOOP FIRST ", Dumper $first;
 #warn "     MORE  ", Dumper $t;
 		if($first->isa('MF::PARENS'))
 		{	my $level = $first->level;
@@ -154,7 +152,6 @@ sub _build_ast($$)
 			my $next  = $self->_build_ast($t, $prio)
 				or error __x"expression '{name}', monadic '{op}' not followed by anything useful",
 				    name => $self->name, op => $op;
-#warn "HERE";
 
 			$first = MF::PREFIX->new($op, $next);
 			redo PROGRESS;
@@ -177,7 +174,23 @@ sub _build_ast($$)
 			if $next_prio < $prio
 			|| ($next_prio==$prio && $assoc==MF::OPERATOR::LTR);
 
+		if($op eq ':')
+		{	return $first;
+		}
+
 		shift @$t;    # apply the operator
+		if($op eq '?')
+		{	my $then  = $self->_build_ast($t, 0);
+			my $colon = shift @$t;
+			$colon && $colon->token eq ':'
+				or error __x"expression '{name}', expected ':' in '?:', but got '{token}'",
+					name => $self->name, token => ($next ? $colon->token : 'end-of-line');
+
+			my $else = $self->_build_ast($t, $next_prio);
+			$first = MF::TERNARY->new($op, $first, $then, $else);
+			redo PROGRESS;
+		}
+
 		$first = MF::INFIX->new($op, $first, $self->_build_ast($t, $next_prio));
 		redo PROGRESS;
 	}
