@@ -5,7 +5,7 @@
 
 package Math::Formula::Context;
 use vars '$VERSION';
-$VERSION = '0.11';
+$VERSION = '0.12';
 
 
 use warnings;
@@ -31,13 +31,16 @@ sub init($)
 	my $name   = $self->{MFC_name}   = $args->{name} or error __x"context without a name";
 
 	$self->{MFC_attrs} = {
-		name       => MF::STRING->new($name),
-		mf_version => MF::STRING->new(undef, $Math::Formula::VERSION),
-		version    => $args->{version} ? MF::STRING->new($args->{version}) : undef,
-		created    => $self->_default(created => 'MF::DATETIME', $args->{created}, DateTime->now),
+		ctx_name       => MF::STRING->new($name),
+		ctx_version    => $args->{version}    ? MF::STRING->new($args->{version}) : undef,
+		ctx_created    => $self->_default(created => 'MF::DATETIME', $args->{created}, DateTime->now),
+		ctx_updated    => $args->{updated} ? MF::DATETIME->new(updated => $args->{updated}) : undef,
+		ctx_mf_version => MF::STRING->new(undef, $args->{mf_version} // $Math::Formula::VERSION),
 	};
 
-	if(my $forms = $args->{formula})
+	$self->{MFC_forms}  = { };
+	$self->{MFC_frags}  = { };
+	if(my $forms = $args->{formulas})
 	{	$self->add(ref $forms eq 'ARRAY' ? @$forms : $forms);
 	}
 
@@ -45,6 +48,15 @@ sub init($)
 	$self;
 }
 
+# For save()
+sub _index()
+{	my $self = shift;
+	 +{	attributes => $self->{MFC_attrs},
+		formulas   => $self->{MFC_forms},
+		fragments  => $self->{MFC_frags},
+	  };
+}
+
 #--------------
 
 sub name   { $_[0]->{MFC_name} }
@@ -57,6 +69,7 @@ sub attribute($)
 	Math::Formula->new($name => $def);
 }
 
+#--------------
 #XXX example with fragment
 
 sub add(@)
@@ -124,7 +137,7 @@ sub evaluate($$%)
 	# Wow, I am impressed!  Caused by prefix(#,.) -> infix
 	length $name or return $self;
 
-	my $form = $self->attribute($name) || $self->formula($name);
+	my $form = $name =~ /^ctx_/ ? $self->attribute($name) : $self->formula($name);
 	unless($form)
 	{	warning __x"no formula '{name}' in {context}", name => $name, context => $self->name;
 		return undef;
