@@ -7,7 +7,7 @@ use strict;
 
 package Math::Formula::Token;
 use vars '$VERSION';
-$VERSION = '0.11';
+$VERSION = '0.12';
 
 
 #!!! The declarations of all other packages in this file are indented to avoid
@@ -45,6 +45,7 @@ package
 	MF::OPERATOR;
 
 use base 'Math::Formula::Token';
+use Log::Report 'math-formula', import => [ 'panic' ];
 
 use constant {
     # Associativity
@@ -55,15 +56,19 @@ use constant {
 # "accidentally" is the same value as the M<token()> method produces.
 sub operator() { $_[0][0] }
 
-sub _compute { die }  # must be extended
+sub _compute
+{	my ($self, $context, $expr) = @_;
+	panic +(ref $self) . ' does not compute';
+}
 
 my %table;
 {
 	# Prefix operators and parenthesis are not needed here
     # Keep in sync with the table in Math::Formula
 	my @order = (
-#		[ LTR,     ',' ],   ? :
-		[ LTR,     qw/or xor/ ],
+#		[ LTR,     ',' ],
+ 		[ LTR,     '?', ':' ],        # ternary ?:
+		[ LTR,     qw/or xor/, '//' ],
 		[ LTR,     'and' ],
 		[ NOCHAIN, qw/ <=> < <= == != >= > / ],
 		[ NOCHAIN, qw/ cmp lt le eq ne ge gt/ ],
@@ -89,7 +94,7 @@ my %table;
 # LTR (compute left to right), RTL (right to left), or NOCHAIN (non-stackable
 # operator).
 
-sub find($) { @{$table{$_[1]} // die "op $_[1]" } }
+sub find($) { @{$table{$_[1]} // panic "op $_[1]" } }
 
 #-------------------
 # MF::PREFIX, monadic prefix operator
@@ -109,8 +114,7 @@ sub _compute($$)
     my $value = $self->child->_compute($context, $expr)
 		or return undef;
 
-	# no prefix operator needs $context or $expr yet
-	$value->prefix($self->operator);
+	$value->prefix($self->operator, $context);
 }
 
 #-------------------
@@ -169,12 +173,28 @@ sub _compute($$)
 	$left->infix($op, $right, $context, $expr);
 }
 
+
 #-------------------
-# MF::FORMULAR, temporary wrapper for a formular
-# When a NAME has been looked-up at the left side of an infix operator, or as
-# child of a prefix operator, it will become this.
+# MF::TERNARY,  if ? then : else
+# Ternary operators have three arguments.  This is a specialization from the
+# MF::OPERATOR type, hence shares its methods.
 
 package
-	MF::Operator;
+	MF::TERNARY;
+
+use base 'MF::OPERATOR';
+
+sub condition() { $_[0][1] }
+sub then()      { $_[0][2] }
+sub else()      { $_[0][3] }
+
+sub _compute($$)
+{	my ($self, $context, $expr) = @_;
+
+    my $cond  = $self->condition->_compute($context, $expr)
+		or return undef;
+
+	($cond->value ? $self->then : $self->else)->_compute($context, $expr)
+}
 
 1;
