@@ -8,7 +8,7 @@ use v5.16;  # fc
 
 package Math::Formula::Type;
 use vars '$VERSION';
-$VERSION = '0.11';
+$VERSION = '0.12';
 
 use base 'Math::Formula::Token';
 
@@ -47,11 +47,11 @@ sub _value { $_[1] }
 
 sub collapsed($) { $_[0]->token =~ s/\s+/ /gr =~ s/^ //r =~ s/ $//r }
 
-sub prefix
-{   my ($self, $op, $child) = @_;
+sub prefix()
+{   my ($self, $op, $context) = @_;
 
-	error __x"cannot find prefx operator '{op}' on a ({child})",
-		op => $op, child => $child;
+	error __x"cannot find prefx operator '{op}' on a {child}",
+		op => $op, child => ref $self;
 }
 
 sub attribute {
@@ -86,11 +86,11 @@ package
 use base 'Math::Formula::Type';
 
 sub prefix($)
-{	my ($self, $op) = @_;
+{	my ($self, $op, $context) = @_;
 	if($op eq 'not')
 	{	return MF::BOOLEAN->new(undef, ! $self->value);
 	}
-	$self->SUPER::prefix($op);
+	$self->SUPER::prefix($op, $context);
 }
 
 sub infix($$$)
@@ -177,8 +177,9 @@ sub infix($$$)
 }
 
 my %string_attrs = (
-   length   => sub { MF::INTEGER->new(undef, length($_[0]->value))  },
-   lower    => sub { MF::STRING->new(undef, fc($_[0]->value)) },
+	length   => sub { MF::INTEGER->new(undef, length($_[0]->value))  },
+	is_empty => sub { MF::BOOLEAN->new(undef, $_[0]->value !~ m/\P{Whitespace}/) },
+	lower    => sub { MF::STRING->new(undef, fc($_[0]->value)) },
 );
 
 sub attribute($) { $string_attrs{$_[1]} || $_[0]->SUPER::attribute($_[1]) }
@@ -199,10 +200,10 @@ sub cast($)
 }
 
 sub prefix($)
-{	my ($self, $op) = @_;
+{	my ($self, $op, $context) = @_;
 	  $op eq '+' ? $self
 	: $op eq '-' ? MF::INTEGER->new(undef, - $self->value)
-	: $self->SUPER::prefix($op)
+	: $self->SUPER::prefix($op, $context);
 }
 
 sub infix($$$)
@@ -274,11 +275,11 @@ sub cast($)
 	: $self->SUPER::cast($to);
 }
 
-sub prefix($)
-{	my ($self, $op) = @_;
+sub prefix($$)
+{	my ($self, $op, $context) = @_;
 	  $op eq '+' ? $self
 	: $op eq '-' ? MF::FLOAT->new(undef, - $self->value)
-	: $self->SUPER::prefix($op)
+	: $self->SUPER::prefix($op, $context)
 }
 
 sub infix($$$)
@@ -602,11 +603,11 @@ sub _value($)
 	$negative ? $duration->multiply(-1) : $duration;
 }
 
-sub prefix($)
-{   my ($self, $op, $right) = @_;
+sub prefix($$)
+{   my ($self, $op, $context) = @_;
 		$op eq '+' ? $self
 	  : $op eq '-' ? MF::DURATION->new('-' . $self->token)
-	  : $self->SUPER::prefix($op)
+	  : $self->SUPER::prefix($op, $context);
 }
 
 sub infix($$@)
@@ -669,6 +670,15 @@ sub cast(@)
 	$context->evaluate($self->token, expect => $type);
 }
 
+sub prefix($$)
+{	my ($self, $op, $context) = @_;
+
+	return MF::BOOLEAN->new(undef, defined $context->formula($self->token))
+		if $op eq 'exists';
+
+	$self->SUPER::prefix($op, $context);
+}
+
 sub infix(@)
 {	my $self = shift;
 	my ($op, $right, $context) = @_;
@@ -678,15 +688,23 @@ sub infix(@)
 	{	my $left = $name eq '' ? MF::FRAGMENT->new($context->name, $context) : $context->evaluate($name);
 		return $left->infix(@_) if $left;
 	}
+
 	if($op eq '#')
 	{	my $left = $name eq '' ? MF::FRAGMENT->new($context->name, $context) : $context->fragment($name);
 		return $left->infix(@_) if $left;
 	}
 
+	if($op eq '//')
+	{	return defined $context->formula($name)
+		  ? $context->evaluate($name)
+		  : $right->_compute($context);
+	}
+
 	my $left = $context->evaluate($name);
 	$left ? $left->infix($op, $right, $context): undef;
 }
 
+
 #-----------------
 
 package
